#include "board.h"		// Class declaration

// Constructor

Board :: Board():
	width(18.0f),
	height(15.0f),
	level(1)
{ }

// Destructor

Board :: ~Board()
{ }

// Loads any extra data not able to be done at constructor time

Board :: LoadData()
{
	ball_bounce = FSOUND_Sample_Load(FSOUND_FREE, "Data/brick_bounce.wav", FSOUND_2D, 0);

	permanent_brick_texture.LoadBMP("Data/permanent_brick.bmp", GL_LINEAR, GL_LINEAR);
	brick_texture.LoadBMP("Data/brick1.bmp", GL_LINEAR, GL_LINEAR);  // Load brick texture
}

// Start a new game by loading level 1 again.

Board :: NewGame()
{
	level = 1;
	LoadLevel(level);
	score = 0;
}

// Go to the next level and load the board

bool Board :: NextLevel()
{
	if (!LoadLevel(++level))					// If we get false for the next level, it means game over
		return false;							// Return false (No more levels)
	return true;								// Otherwise return true - everything went ok
}


// Load specified level and initialize bricks to what we want
// The Level file is a 12x15x24bit Windows Bitmap file. - Or can be respecified with NUM_BRICKS_VERTICAL and NUM_OF_BRICKS_HORIZONTAL
// * White bricks (255,255,255) represent no brick
// * Black bricks (0,0,0) represent bricks that cant be removed
// Returns false if we dont have any level specified - either a stuff up or GAME OVER!
// Thanks to NeHe for the BMP loader that I modified for this.

bool Board :: LoadLevel(int level)
{
	char *filename;
	FILE* file;									// The file pointer
	BITMAPFILEHEADER	file_header;			// The bitmap file header
	BITMAPINFOHEADER	info_header;
	int					imageIdx = 0;			// Image index counter
	unsigned char		tempRGB;				// Swap variable
	GLubyte* data;								// Image data (up to 32 bits)
	unsigned int bpp;							// Image color depth in bits per pixel
	unsigned int width;							// Image width
	unsigned int height;						// Image height

	bricks_left = 0;							// Reset amount of bricks in level

	// Decide which level to load, depending on given level
	switch (level)
	{
		case 1:
			filename = "Data/Level1.dat";
			floor.LoadBMP("Data/floor1.bmp", GL_LINEAR, GL_LINEAR);			// The floor texture
			wall.LoadBMP("Data/wall1.bmp", GL_LINEAR, GL_LINEAR);			// The side wall texture
			InitBricks();						// Call InitBricks(), to reset the loaded textures of all the bricks
			break;
		case 2:
			filename = "Data/Level2.dat";
			floor.LoadBMP("Data/floor2.bmp", GL_LINEAR, GL_LINEAR);			// The floor texture
			wall.LoadBMP("Data/wall1.bmp", GL_LINEAR, GL_LINEAR);			// The side wall texture
			InitBricks();						// Call InitBricks(), to reset the loaded textures of all the bricks
			break;
		case 3:
			filename = "Data/Level3.dat";
			floor.LoadBMP("Data/floor3.bmp", GL_LINEAR, GL_LINEAR);			// The floor texture
			wall.LoadBMP("Data/wall1.bmp", GL_LINEAR, GL_LINEAR);			// The side wall texture
			InitBricks();						// Call InitBricks(), to reset the loaded textures of all the bricks
			break;
		case 4:
			filename = "Data/Level4.dat";
			floor.LoadBMP("Data/floor4.bmp", GL_LINEAR, GL_LINEAR);			// The floor texture
			wall.LoadBMP("Data/wall1.bmp", GL_LINEAR, GL_LINEAR);			// The side wall texture
			InitBricks();						// Call InitBricks(), to reset the loaded textures of all the bricks
			break;
		default:
			return false;						// Return false if we don't have the next level here - GAMEOVER!
	}

	// Open filename in read only binary mode
	file= fopen(filename, "rb");

	// Read the bitmap file header
	fread(&file_header, sizeof(BITMAPFILEHEADER), 1, file);
	
	// Confirm that this is a bitmap by checking for the universal bitmap id
	if(file_header.bfType != 0x4D42)
	{
		fclose(file);
	}

	// Read the bitmap information header in
	fread(&info_header, sizeof(BITMAPINFOHEADER), 1, file);

	// Advance the file pointer to the beginning of the bitmap data
	fseek(file, file_header.bfOffBits, SEEK_SET);

	// Allocate the bitmap image data
	data = new unsigned char [info_header.biSizeImage];

	// Confirm memory allocation
	if(!data)
	{
		free(data);
		fclose(file);
	}

	// Read in the bitmap image data
	fread(data, 1, info_header.biSizeImage, file);

	// Make sure bitmap image data was read
	if(data==NULL)
	{
		fclose(file);
	}
	
	width = info_header.biWidth;
	height= info_header.biWidth;
	bpp	  = info_header.biBitCount;

	int y=0, x=0;									// Initialize the brick counters

	// Swap the R and B values to get RGB since the bitmap color format is in BGR
	for(imageIdx = 0; imageIdx<(int)info_header.biSizeImage; imageIdx+=3)
	{
		tempRGB			= data[imageIdx];
		data[imageIdx]  = data[imageIdx + 2];
		data[imageIdx+2]= tempRGB;

		bricks[y][x].color[0] = data[imageIdx]/255.0f; // Set the RGB properties of each brick (divide by 255 since we get an 8bit byte for each colour channel, but we want to give OpenGL a value between 0 and 1
		bricks[y][x].color[1] = data[imageIdx + 1]/255.0f;
		bricks[y][x].color[2] = data[imageIdx + 2]/255.0f;

		bricks[y][x].status = BRICK_ACTIVE;			// Set brick status to active by default
		bricks[y][x].hits = 1;						// Set the number of brick hits to 1 by default
		bricks[y][x].texture = brick_texture.ID;	// Set regular texture by default

		if ((data[imageIdx] == 255) &&				// White represents no brick
			(data[imageIdx+1] == 255) &&
			(data[imageIdx+2] == 255))
			bricks[y][x].status = BRICK_DESTROYED;	// Disable white bricks

		else if ((data[imageIdx] == 0) &&			// Black represents a permanent brick
			(data[imageIdx+1] == 0) &&
			(data[imageIdx+2] == 0))
		{
			bricks[y][x].status = BRICK_PERMANENT;	// Black bricks cannot be destroyed
			bricks[y][x].color[0] = 1;				// Brick to white
			bricks[y][x].color[1] = 1;
			bricks[y][x].color[2] = 1;
			bricks[y][x].texture = permanent_brick_texture.ID; // Set texture to other permanent looking brick
		}
		else
			bricks_left++;							// Increment the amount of bricks if the brick is not active

		if (++x >= NUM_BRICKS_HORIZONTAL)			// Check if we need to go to the next line in the bricks array
		{
			x=0;
			y++;
		}
	}

	fclose(file);									// Close the file

	return true;									// Everything went ok!
}

// Initializes array of bricks
// Sets texture and bounce sound + width and height

Board :: InitBricks()
{
	for (int y=0; y<NUM_BRICKS_VERTICAL; y++)
		for (int x=0; x<NUM_BRICKS_HORIZONTAL; x++)
		{
			bricks[y][x].x = x*bricks[y][x].width - width/2;
			bricks[y][x].y = y*bricks[y][x].height - NUM_BRICKS_VERTICAL * bricks[y][x].height + height/2 - bricks[y][x].height;
			bricks[y][x].sound_ball_bounce = ball_bounce;		// Set the ball bouncing off brick sound
		}
}

// Checks ball collisions with bricks

Board :: Check_Brick_Collisions(Ball *ball)
{
	for (int y=0; y<NUM_BRICKS_VERTICAL; y++)
		for (int x=0; x<NUM_BRICKS_HORIZONTAL; x++)
			if (bricks[y][x].Check_Brick_Ball_Collision(ball))
			{
				score += 10;									// Increment the score
				--bricks_left;									// Decrement bricks remaining
				FSOUND_PlaySound(FSOUND_FREE, bricks[y][x].sound_ball_bounce); // Play sound of collision with ball
			}
}

// Draw to opengl screen

Board :: Draw()
{
	glColor4f(1, 1, 1, 1);
	glBindTexture(GL_TEXTURE_2D, floor.ID);						// Bind floor data texture
	glEnable(GL_TEXTURE_2D);
	glBegin(GL_QUADS);
	  glTexCoord2f(0,0); glVertex3f(-width/2, -height/2, 0.0f);	// Draw floor
	  glTexCoord2f(0,1); glVertex3f(-width/2,  height/2, 0.0f);
	  glTexCoord2f(1,1); glVertex3f( width/2,  height/2, 0.0f);
	  glTexCoord2f(1,0); glVertex3f( width/2, -height/2, 0.0f);
	glEnd();

	for (int y=0; y<NUM_BRICKS_VERTICAL; y++)
		for (int x=0; x<NUM_BRICKS_HORIZONTAL; x++)
		{
			bricks[y][x].Draw();								// Draw the array of bricks
		}

	glColor4f(1, 1, 1, 1);
	glBindTexture(GL_TEXTURE_2D, wall.ID);						// Bind floor data texture

	glBegin(GL_QUADS);

	  glTexCoord2f(0,0); glVertex3f(-width/2-0.5f,	-height/2,	   0.5f);	// draw left wall - top edge
	  glTexCoord2f(0,1); glVertex3f(-width/2,		-height/2,	   0.5f);
	  glTexCoord2f(1,1); glVertex3f(-width/2,		 height/2+0.5f, 0.5f);
	  glTexCoord2f(1,0); glVertex3f(-width/2-0.5f,   height/2+0.5f, 0.5f);

	  glTexCoord2f(0,0); glVertex3f(-width/2-0.5f,	-height/2,		0.0f);	// draw left wall - left edge
	  glTexCoord2f(0,1); glVertex3f(-width/2-0.5f,	 height/2+0.5f, 0.0f);
	  glTexCoord2f(1,1); glVertex3f(-width/2-0.5f,   height/2+0.5f, 0.5f);
	  glTexCoord2f(1,0); glVertex3f(-width/2-0.5f,	-height/2,		0.5f);

	  glTexCoord2f(0,0); glVertex3f(-width/2,		-height/2,		0.0f);	// draw left wall - right edge
	  glTexCoord2f(0,1); glVertex3f(-width/2,		 height/2+0.5f, 0.0f);
	  glTexCoord2f(1,1); glVertex3f(-width/2,		 height/2+0.5f, 0.5f);
	  glTexCoord2f(1,0); glVertex3f(-width/2,		-height/2,		0.5f);

	  glTexCoord2f(0,0); glVertex3f(-width/2,		 height/2+0.5f,	0.0f);	// draw left wall - top end
	  glTexCoord2f(0,1); glVertex3f(-width/2-0.5f,	 height/2+0.5f, 0.0f);
	  glTexCoord2f(1,1); glVertex3f(-width/2-0.5f,	 height/2+0.5f, 0.5f);
	  glTexCoord2f(1,0); glVertex3f(-width/2,		 height/2+0.5f,	0.5f);

	  glTexCoord2f(0,0); glVertex3f(-width/2-0.5f,	-height/2,		0.0f);	// draw left wall - bottom end
	  glTexCoord2f(0,1); glVertex3f(-width/2-0.5f,	-height/2,		0.5f);
	  glTexCoord2f(1,1); glVertex3f(-width/2,		-height/2,		0.5f);
	  glTexCoord2f(1,0); glVertex3f(-width/2,		-height/2,		0.0f);

	  glTexCoord2f(0,0); glVertex3f( width/2+0.5f,	-height/2,		0.5f);	// draw right wall - top edge
	  glTexCoord2f(0,1); glVertex3f( width/2,	    -height/2,		0.5f);
	  glTexCoord2f(1,1); glVertex3f( width/2,	     height/2+0.5f, 0.5f);
	  glTexCoord2f(1,0); glVertex3f( width/2+0.5f,  height/2+0.5f, 0.5f);

	  glTexCoord2f(0,0); glVertex3f( width/2+0.5f,	-height/2,		0.0f);	// draw right wall - left edge
	  glTexCoord2f(0,1); glVertex3f( width/2+0.5f,	 height/2+0.5f, 0.0f);
	  glTexCoord2f(1,1); glVertex3f( width/2+0.5f,	 height/2+0.5f, 0.5f);
	  glTexCoord2f(1,0); glVertex3f( width/2+0.5f, -height/2,		0.5f);

	  glTexCoord2f(0,0); glVertex3f( width/2,		-height/2,		0.0f);	// draw right wall - right edge
	  glTexCoord2f(0,1); glVertex3f( width/2,		 height/2+0.5f, 0.0f);
	  glTexCoord2f(1,1); glVertex3f( width/2,		 height/2+0.5f, 0.5f);
	  glTexCoord2f(1,0); glVertex3f( width/2,		-height/2,		0.5f);

	  glTexCoord2f(0,0); glVertex3f( width/2+0.5f,	 height/2+0.5f,	0.0f);	// draw right wall - top end
	  glTexCoord2f(0,1); glVertex3f( width/2,		 height/2+0.5f, 0.0f);
	  glTexCoord2f(1,1); glVertex3f( width/2,		 height/2+0.5f, 0.5f);
	  glTexCoord2f(1,0); glVertex3f( width/2+0.5f,	 height/2+0.5f,	0.5f);

	  glTexCoord2f(0,0); glVertex3f( width/2,		-height/2,		0.0f);	// draw right wall - bottom end
	  glTexCoord2f(0,1); glVertex3f( width/2,		-height/2,		0.5f);
	  glTexCoord2f(1,1); glVertex3f( width/2+0.5f,	-height/2,		0.5f);
	  glTexCoord2f(1,0); glVertex3f( width/2+0.5f,	-height/2,		0.0f);

	  glTexCoord2f(0,0); glVertex3f(-width/2,	height/2, 0.0f);			// draw top wall - inner edge
	  glTexCoord2f(0,1); glVertex3f(-width/2,  height/2, 0.5f);
	  glTexCoord2f(1,1); glVertex3f( width/2,  height/2, 0.5f);
	  glTexCoord2f(1,0); glVertex3f( width/2,	height/2, 0.0f);

	  glTexCoord2f(0,0); glVertex3f(-width/2, height/2+0.5f, 0.0f);		// draw top wall - outer edge
	  glTexCoord2f(0,1); glVertex3f(-width/2, height/2+0.5f, 0.5f);
	  glTexCoord2f(1,1); glVertex3f( width/2, height/2+0.5f, 0.5f);
	  glTexCoord2f(1,0); glVertex3f( width/2, height/2+0.5f, 0.0f);
	  
	  glTexCoord2f(0,0); glVertex3f(-width/2, height/2+0.5f, 0.5f);		// draw top wall - top edge
	  glTexCoord2f(0,1); glVertex3f(-width/2, height/2,      0.5f);
	  glTexCoord2f(1,1); glVertex3f( width/2, height/2,      0.5f);
	  glTexCoord2f(1,0); glVertex3f( width/2, height/2+0.5f, 0.5f);

	  glTexCoord2f(0,0); glVertex3f(-width/2, height/2,      0.0f);		// draw top wall - left end
	  glTexCoord2f(0,1); glVertex3f(-width/2, height/2,      0.5f);
	  glTexCoord2f(1,1); glVertex3f(-width/2, height/2+0.5f, 0.5f);
	  glTexCoord2f(1,0); glVertex3f(-width/2, height/2+0.5f, 0.0f);

	  glTexCoord2f(0,0); glVertex3f( width/2, height/2+0.5f,  0.0f);		// draw top wall - right end
	  glTexCoord2f(0,1); glVertex3f( width/2, height/2+0.5f,  0.5f);
	  glTexCoord2f(1,1); glVertex3f( width/2, height/2,       0.5f);
	  glTexCoord2f(1,0); glVertex3f( width/2, height/2,       0.0f);

	glEnd();
}

