#include "Brick.h"							// Class declaration

// Constructor

Brick :: Brick():
	width		(1.5f),						// Brick dimensions
	height		(0.5f),
	hits		(1),						// Number of hits before brick is destroyed
	status		(BRICK_ACTIVE),				// Active/Destroyed? as defined
	texture		(NULL)						// No current texture
{
	color[0] = 1.0f;						// RGB color set
	color[1] = 0.0f;
	color[2] = 0.0f;
}

// Destructor

Brick :: ~Brick()
{ }

//------------------------------------------------------------------//
//- bool Check_Brick_Ball_Collision(Ball *ball) --------------------//
//------------------------------------------------------------------//
//- Description: This function checks if ball has the brick, then  -//
//-              changes the ball direction. If collision occurs   -//
//-              then we return true, otherwise false.			   -//
//------------------------------------------------------------------//

bool Brick :: Check_Brick_Ball_Collision(Ball *ball)
{
	if (status != BRICK_DESTROYED)
	{
		if ((ball->xi > 0) &&									// If ball is travelling right
			((ball->x + ball->diameter + ball->speed) > x) &&	// Check ball will actually cross the path of the paddle during next move
			((ball->x + ball->diameter + ball->speed) < (x + width)) &&
			((ball->y - ball->radius) < y) &&					// Then check we are within the range of the width of the paddle
			((ball->y - ball->radius) >= (y - height)))			// One of these has to be >= otherwise we miss inbetween bricks
		{
			ball->xi = -ball->xi;								// Change x direction
			if (status == BRICK_ACTIVE)
				Hit();
			return true;
		}
		else if ((ball->xi < 0) &&								// If ball is travelling left
			((ball->x - ball->speed) < (x + width)) &&			// Check ball will actually cross the path of the paddle during next move
			((ball->x - ball->speed) > x) &&
			((ball->y - ball->radius) < y) &&					// Then check we are within the range of the width of the paddle
			((ball->y - ball->radius) >= (y - height)))			// One of these has to be >= otherwise we miss inbetween bricks
		{
			ball->xi = -ball->xi;								// Change x direction
			if (status == BRICK_ACTIVE)
				Hit();
			return true;
		}
		if ((ball->yi < 0) &&									// If ball is travelling down
		    (((ball->y - ball->diameter - ball->speed) < y) &&	// Check ball will actually cross the path of the paddle during next move
			((ball->y - ball->diameter - ball->speed) > y - height)) &&
			((ball->x + ball->radius) >= x) &&					// One of these has to be >= otherwise we miss inbetween bricks	
			((ball->x + ball->radius) < (x + width)))			// Then check we are within the range of the width of the paddle
		{
			ball->yi = -ball->yi;
			if (status == BRICK_ACTIVE)
				Hit();
			return true;
		}
		else if ((ball->yi > 0) &&								// If ball is travelling up
			((ball->y + ball->speed) > (y - height)) &&			// Check ball will actually cross the path of the paddle during next move
			((ball->y + ball->speed) < (y)) &&					// But not past the back of it
			((ball->x + ball->radius) >= x) &&					// One of these has to be >= otherwise we miss inbetween bricks
			((ball->x + ball->radius) < (x + width)))			// Then check we are within the range of the width of the paddle
		{
			ball->yi = -ball->yi;
			if (status == BRICK_ACTIVE)
				Hit();
			return true;
		}
	}
	return false;
}

Brick :: Hit()
{
	FSOUND_PlaySound(FSOUND_FREE, sound_ball_bounce);	// play sound of ball hitting brick
	if (--hits == 0 && status != BRICK_PERMANENT)
		status = BRICK_DESTROYED;
}

// Draw to opengl screen

Brick :: Draw()
{
	if (status != BRICK_DESTROYED)				// No need to draw it if it's not active
	{
		glColor3fv(color);						// Set the brick color
		glPushMatrix();							// Save current matrix, so translation doesnt affect rest of program

		glBindTexture(GL_TEXTURE_2D, texture);

		glBegin(GL_QUADS);						// Draw it!
		// Bottom
		  glVertex3f(x,			y		  , 0.0f);
		  glVertex3f(x,			y - height, 0.0f);
		  glVertex3f(x + width, y - height, 0.0f);
		  glVertex3f(x + width, y		  , 0.0f);
		// Top
		  glTexCoord2f(0,1); glVertex3f(x,		   y		 , height);
		  glTexCoord2f(0,0); glVertex3f(x,		   y - height, height);
		  glTexCoord2f(1,0); glVertex3f(x + width, y - height, height);
		  glTexCoord2f(1,1); glVertex3f(x + width, y		 , height);
		// Left side
		  glVertex3f(x,			y		  , 0.0f);
		  glVertex3f(x,			y - height, 0.0f);
		  glVertex3f(x,         y - height, height);
		  glVertex3f(x,         y		  , height);
		// Right side
		  glVertex3f(x + width, y		  , 0.0f);
		  glVertex3f(x + width, y - height, 0.0f);
		  glVertex3f(x + width, y - height, height);
		  glVertex3f(x + width, y		  , height);
		// Top end
		  glVertex3f(x + width, y		  , 0.0f);
		  glVertex3f(x,			y		  , 0.0f);
		  glVertex3f(x,			y		  , height);
		  glVertex3f(x + width, y		  , height);
		// Bottom end
		  glTexCoord2f(0,0); glVertex3f(x,		   y - height, 0.0f);
		  glTexCoord2f(1,0); glVertex3f(x + width, y - height, 0.0f);
		  glTexCoord2f(1,1); glVertex3f(x + width, y - height, height);
		  glTexCoord2f(0,1); glVertex3f(x,		   y - height, height);
		glEnd();

		glPopMatrix();								// Restore matrix
	}
}

