//==================================================================//
//==================================================================//
//= DirectInput.cpp ================================================//
//==================================================================//
//= Original coder: Trent Polack (ShiningKnight) ===================//
//==================================================================//
//==================================================================//

#include "Shining3D.h"


//------------------------------------------------------------------//
//------------------------------------------------------------------//
//- GLOBALS --------------------------------------------------------//
//------------------------------------------------------------------//
//------------------------------------------------------------------//
DIMOUSESTATE  mouseState;	//The mouse's 'buffer'
UCHAR keyBuffer[256];		//The keyboard's key buffer


//------------------------------------------------------------------//
//- bool SHININGINPUT::Init(void) ----------------------------------//
//------------------------------------------------------------------//
//- Description: Initiates DirectInput (with the keyboard).  Will  -//
//-              return true if everything goes ok, and false if   -//
//-              something goes wrong.							   -//
//------------------------------------------------------------------//
bool SHININGINPUT::
	Init(void)
	{
	HRESULT hr;	//DirectX result variable

	//Create the main DirectInput object
	hr= (DirectInput8Create(hinstance, DIRECTINPUT_VERSION,
						   IID_IDirectInput8, (void**)&lpdi, NULL));
	if(FAILED(hr))
		{
		MessageBox(hwnd, "Could not create main DInput object", "ERROR", MB_OK);
		return false;
		}

	//Create the keyboard's device object
	hr= (lpdi->CreateDevice(GUID_SysKeyboard, &lpdiKeyboard, NULL));
	if(FAILED(hr))
		{
		MessageBox(hwnd, "Could not create keyboard's object",  "ERROR",MB_OK);
		Shutdown();
		return false;
		}

	//Set the keyboard's data format 
	hr= (lpdiKeyboard->SetDataFormat(&c_dfDIKeyboard));
	if(FAILED(hr))
		{
		MessageBox(hwnd, "Could not set keyboard's data format","ERROR",MB_OK);
		Shutdown();
		return false;
		}

	//Set the keyboard's cooperation level with your computer's
	//environment.
	hr= (lpdiKeyboard->SetCooperativeLevel(hwnd, DISCL_BACKGROUND|DISCL_NONEXCLUSIVE));
	if(FAILED(hr))
		{
		MessageBox(hwnd, "Could not set keyboard's cooperation level", "ERROR", MB_OK);
		Shutdown();
		return false;
		}

	//And finally, acquire the keyboard for use.
	lpdiKeyboard->Acquire();

	//Its time to get mousey
	hr= lpdi->CreateDevice(GUID_SysMouse, &lpdiMouse, NULL);
	if(FAILED(hr))
		{
		MessageBox(hwnd, "Could not set create the mouse device", "ERROR", MB_OK);
		Shutdown();
		return FALSE;
		}

	//Set the mouse's data format
	hr= lpdiMouse->SetDataFormat(&c_dfDIMouse);
	if(FAILED(hr)) 
		{
		MessageBox(hwnd, "Could not set the mouse's data format", "ERROR", MB_OK);
		Shutdown();
		return FALSE;
		}

	//Set the mouse's cooperative level with that of windows
	hr= lpdiMouse->SetCooperativeLevel(hwnd, DISCL_BACKGROUND | DISCL_NONEXCLUSIVE);
	if(FAILED(hr)) 
		{
		MessageBox(hwnd, "Could not set the mouse's behavior", "ERROR", MB_OK);
		Shutdown();
		return FALSE;
		}

	//Acquire the mouse
	hr= lpdiMouse->Acquire();
	if(FAILED(hr))
		{
		MessageBox(hwnd, "Could not acquire the mouse", "ERROR", MB_OK);
		Shutdown();
		}

	//Set the mouse's device state
	hr= lpdiMouse->GetDeviceState(sizeof(DIMOUSESTATE), &mouseState);
	if(FAILED(hr))
		{
		MessageBox(hwnd, "The mouse has been lost in initialization", "ERROR", MB_OK);
		Shutdown();
		}

	//Time to rock and roll
	return true;
	}
	
	
//------------------------------------------------------------------//
//- void SHININGINPUT::Shutdown(void) ------------------------------//
//------------------------------------------------------------------//
//- This function shuts down DirectInput.  This  function is	   -//
//- automatically called if there is an error in the initiation	   -//
//- sequence, but if there was no error, you will need to call it  -//
//- from your shutdown function.								   -//
//------------------------------------------------------------------//	
void SHININGINPUT::
	Shutdown(void)
	{
	//First, since we create the mouse last, we need to 'destroy'
	//it first.
	if(lpdiMouse!=NULL)
		{
		//Unacquire the mouse
		lpdiMouse->Unacquire();

		//Throw the mouse away
		lpdiMouse->Release();
		lpdiMouse= NULL;
		}

	if(lpdiKeyboard!=NULL)
		{
		//Unacquire the keyboard
		lpdiKeyboard->Unacquire();
		
		//Let the keyboard go faster than your ex-girlfriend's phone number
		lpdiKeyboard->Release();
		lpdiKeyboard= NULL;
		}
	
	//Now the main DirectInput object
	if(lpdi!=NULL)
		{
		//Kill, burn, and destroy the main object
		lpdi->Release();
		lpdi=NULL;
		}
	}


//------------------------------------------------------------------//
//- void SHININGINPUT::Update(void) --------------------------------//
//------------------------------------------------------------------//
//- This function updates DirectInput to make sure that your	   -//
//- program still has access to the keyboard and if it doesn't, the-//
//- function tries to reacquire it.  If not, the function calls	   -//
//- Shutdown().													   -//
//------------------------------------------------------------------//
void SHININGINPUT::
	Update(void)
	{
	HRESULT hr;
	
	//First, check to see if the keyboard is still working/functioning
    hr= (lpdiKeyboard->GetDeviceState(sizeof(UCHAR[256]),(LPVOID)&keyBuffer));
    if(FAILED(hr))
		{ 
        if(hr==DIERR_INPUTLOST) 
			{
			//Try to re-acquire the keyboard
			hr= (lpdiKeyboard->Acquire());
			if(FAILED(hr))
				{
				MessageBox(hwnd, "Keyboard has been lost", "ERROR", MB_OK);
				Shutdown();
				}
			}
		}

	//Now check the mouse
    hr= (lpdiMouse->GetDeviceState(sizeof(DIMOUSESTATE),(LPVOID)&mouseState));
    if(FAILED(hr))
		{ 
        if(hr==DIERR_INPUTLOST) 
			{
			//Try to re-acquire the keyboard
			hr= (lpdiMouse->Acquire());
			if(FAILED(hr))
				{
				MessageBox(hwnd, "The mouse has been lost", "ERROR", MB_OK);
				Shutdown();
				}
			}
		}

	GetWheelMovement();
	GetMouseMovement();
	}

void SHININGINPUT::
	GetWheelMovement(void)
	{	z= mouseState.lZ;	}

void SHININGINPUT::
	GetMouseMovement(void)
	{
	x= mouseState.lX;
	y= mouseState.lY;
	}