// Breakout
//
// Author: Joshua Jenkins
// Homepage: http://homepages.tig.com.au/~grom
// Email: grom@tig.com.au
//
// Your free to look at my work, use parts of it if you wish.
// If you would like to ask a question or give some feedback,
// I would love to hear from you.
//
// By the way: I'm still perfecting this OO stuff, and know
// things could be encapsulated alot better, I'm planning
// in improving things in future programs.
//
// I could improve the collision detection arhcitecture by 
// having a base class that can do abounding box collision between
// itself and another given instance. This would be virtual so
// polymorphism could occur. The paddle, bricks and ball could be
// extensions of this base class

#include <time.h>										// timeGetTime() function
#include <fstream.h>									// File IO (highscores)

#include "fmod.h"										// FMod sound system API

#include "Shining3D.h"									// Main game system

#include "Paddle.h"
#include "Ball.h"
#include "Board.h"
														// Define game states
#define PLAYING			0								// Game in progress
#define	BALL_OUT		1								// Ball has fallen from play area
#define GAME_OVER		2								// Ready to play...space to start
#define GET_READY		3								// Ball is about to come into action
#define GET_READY_TIME	1000							// How long to pause for player to be ready
#define WON				4								// Player has gone through all the levels and WON!

// Globals //

SHINING3D game_system;									// Main game system
SHININGINPUT sinput;									// A bit of direct input

TEXTURE font;											// Space for font texture
TEXTURE ball_texture;									// Texture for the ball

Paddle player;											// Player paddle
Ball ball;												// The ping pong ball
Board board;											// Playing board

int highscore;											// Highscore that is stored after game in highscore.dat
int game_state;											// Current game state - as defined
DWORD game_state_timer;									// Timer for various game_state

FSOUND_SAMPLE *sound_wall_bounce;						// bounce off wall
FSOUND_SAMPLE *sound_ball_falling;						// when ball falls off board (millisecs)

// Function Declarations //

void InitFMOD(void);
bool Game_Init(void);
bool Game_Main(void);
bool Game_Shutdown(void);
void Move_Paddle(void);
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow);

// Function Definitions //

//------------------------------------------------------------------//
//- void InitFMOD(void) --------------------------------------------//
//------------------------------------------------------------------//
//- Description: This function initializes the FMoD sound system,  -//
//-              loads sound effects and starts a song playing.    -//
//------------------------------------------------------------------//

void InitFMOD(void)
{
	FMUSIC_MODULE *song;								// Storage for background music
    if (FSOUND_Init(44100, 32, 0))						// Initialize sound board at 44kHz
	{

		song = FMUSIC_LoadSong("Data/kmheroes.it");		// Load our selected track

		FMUSIC_PlaySong(song);							// Start it playing

		sound_wall_bounce	= FSOUND_Sample_Load(FSOUND_FREE, "data/wall_bounce.wav", FSOUND_2D, 0);
		sound_ball_falling  = FSOUND_Sample_Load(FSOUND_FREE, "data/ball_falling.wav", FSOUND_2D, 0);
	}
}

//------------------------------------------------------------------//
//- bool Game_Init(void) -------------------------------------------//
//------------------------------------------------------------------//
//- Description: This function does all the necesary initiation    -//
//-              that you want done in your game.  It is called    -//
//-              once, and only once.                              -//
//------------------------------------------------------------------//

bool Game_Init(void)
{
	ifstream highscore_file;									// Highscore data file

	GLfloat LightAmbient[] = { 0.5f, 0.5f, 0.5f, 1.0f};			// Ambient light values
	GLfloat LightDiffuse[] = { 1.0f, 1.0f, 1.0f, 1.0f};			// Diffuse light values
	GLfloat LightPosition[] = { 0.0f, 0.0f, 2.0f, 1.0f};		// Light position

	highscore_file.open("Data/highscore.dat");					// Open the highscore file
	highscore_file >> highscore;								// Read in the data
	highscore_file.close();										// Close the file since we are finished with it;

	game_system.OpenGLInit(640,480,16, GIVE_CHOICE, "Breakout");// Create a fullscreen/window with 640x480x16 dimensions
	sinput.Init();												// Initiate DirectInput
	game_system.FontInit();										// Initiate the font

	font.LoadTGA("Data/Font.tga", GL_LINEAR, GL_LINEAR);		// Load font data

	InitFMOD();													// Initialize sound system

	player.LoadData();											// Load any extra things the paddle needs (sound effects)
	player.x = -player.width/2;									// Center paddle on Y-Axis
	player.y = -board.height/2 + 1.5f;

	ball_texture.LoadBMP("Data/ball.bmp", GL_LINEAR, GL_LINEAR);// Load brick texture
	ball.Reset();												// Reset ball

	board.LoadData();											// Load any extra things the board needs
	board.NewGame();											// Reset board to level 1
	player.lives = 3;											// Player starts with 3 lives

	glLightfv(GL_LIGHT0, GL_AMBIENT, LightAmbient);				// Setup ambient light
	glLightfv(GL_LIGHT0, GL_DIFFUSE, LightDiffuse);				// Setup diffuse light
	glLightfv(GL_LIGHT0, GL_POSITION, LightPosition);			// Setup light position
	glEnable(GL_LIGHT0);

	game_state = GAME_OVER;										// Wait until players are ready

	return true;
}

//------------------------------------------------------------------//
//- void Move_Paddle(void) ----------------------------------------//
//------------------------------------------------------------------//
//- Description: This function checks if a key is down and updates -//
//-              paddle appropriately (tests for collisions on	   -//
//-              walls).										   -//
//------------------------------------------------------------------//

void Move_Paddle(void)
{
	player.oldx = player.x; player.oldy = player.y;		// Save old paddle positions

	if(KEY_DOWN(DIK_RIGHT))
		if ((player.x + player.width + player.speed) > board.width/2) // If paddle will end up over the end of board
			player.x = board.width/2 - player.width;	// Then set it stop at board edge
		else
			player.x += player.speed;					// Otherwise move paddle by it's speed
	if(KEY_DOWN(DIK_LEFT))
		if ((player.x - player.speed) < -board.width/2) // If paddle will end up over the end of board
			player.x = -board.width/2;					// Then set it stop at board edge
		else
			player.x -= player.speed;					// Otherwise move paddle by it's speed
}

//------------------------------------------------------------------//
//- bool Game_Main(void) -------------------------------------------//
//------------------------------------------------------------------//
//- Description: This function is the heart of your program/game,  -//
//-              it is called in succession until you return a     -//
//-              false value to it, then the Game_Shutdown()       -//
//-              function is called.                               -//
//------------------------------------------------------------------//

bool Game_Main(void)
{
	static float rot=0;									// Rotation value for swinging board around
	
	sinput.Update();									// Update DirectInput to make sure it's still alive
	mouseState.lX = game_system.SCREEN_WIDTH/2;
	mouseState.lY = game_system.SCREEN_HEIGHT/2;

	if(KEY_DOWN(DIK_1))							// Check to see if the user wants to quit
		board.LoadLevel(1);
	if(KEY_DOWN(DIK_2))							// Check to see if the user wants to quit
		board.LoadLevel(2);
	if(KEY_DOWN(DIK_3))							// Check to see if the user wants to quit
		board.LoadLevel(3);
	if(KEY_DOWN(DIK_4))							// Check to see if the user wants to quit
		board.LoadLevel(4);

	if(KEY_DOWN(DIK_ESCAPE))							// Check to see if the user wants to quit
		PostQuitMessage(0);

	if(KEY_DOWN(DIK_SPACE)) {							// Reset ball on space press
		if (game_state == GAME_OVER)					// If the game is over and space is pressed
		{
			ball.Reset();								// Reset ball and serve
			board.NewGame();							// Reset board (and the score)
			player.lives = 3;							// Player starts with 3 lives
			game_state_timer = timeGetTime() + GET_READY_TIME; // Start timer
			game_state = GET_READY;						// Start the game!
		}
	}

	Move_Paddle();										// Check if paddle needs moving with keyboard input

	// Start of the main game loop update //
	if (game_state == PLAYING || game_state == BALL_OUT)			// Do the updates when playing
	{
		if (board.bricks_left == 0)									// If there are no bricks remaining
		{
			if(!board.NextLevel())									// Go to the next level
				game_state = WON;									// If there are no more levels, then player has WON!
			game_state_timer = timeGetTime() + GET_READY_TIME; // Start timer
			game_state = GET_READY;									// Then pause for player to get ready
			ball.Reset();											// Reset ball
		}

		// Check if ball is going to fall off the edge of board
		if (game_state != BALL_OUT)
		{
			if (((ball.y - ball.diameter - ball.speed) < -board.height/2) && // If ball is going to fall off board
				(ball.z == 0.0f))
			{
				game_state = BALL_OUT;								// Change game state
				FSOUND_PlaySound(FSOUND_FREE, sound_ball_falling);	// play "ball falling" sound
			}
		}

		if (game_state == BALL_OUT)									// Do this if the ball is off the board
		{
			ball.zi -= 0.07f;										// Accelerate ball downwards if it falls off the edge
			if (ball.zi < -7.0f) {									// If ball has fallen for long enough
				ball.Reset();										// Reset ball and serve
				game_state_timer = timeGetTime() + GET_READY_TIME;	// Start timer
				game_state = GET_READY;								// Then reset ball and get ready!
				if (--player.lives == 0)							// If player is out of lives
				{
						if (board.score > highscore)				// Update the highscore if is the best one
						{
							ofstream highscorefile;					// File for output
							highscore = board.score;				// Set the new highscore
							highscorefile.open("Data/highscore.dat"); // Write the highscore data to file
							highscorefile << highscore << endl << board.level;
							highscorefile.close();					// Close the file since we are finished with it
						}
					game_state = GAME_OVER;							// Then it's time to die!
				}
			}
		}

		player.Check_Ball_Collision(&ball);							// Check if ball has hit a player's paddle

		board.Check_Brick_Collisions(&ball);								// Check if ball has hit any bricks

		// Check ball collisions with wall
		if (ball.z == 0.0f)													// If ball isn't falling
		{
			if ((ball.y + ball.speed) > board.height/2)						// If ball is going to hit top wall
			{
				ball.yi *= -1;												// Then change it's direction
				FSOUND_PlaySound(FSOUND_FREE, sound_wall_bounce);			// play "wall bounce" sound
			}
			if (((ball.x + ball.diameter + ball.speed) > board.width/2) ||	// If ball is going to hit right or left wall
				((ball.x - ball.speed) < -board.width/2))
			{
				ball.xi *= -1;
				FSOUND_PlaySound(FSOUND_FREE, sound_wall_bounce);			// play "wall bounce" sound
			}
		}
	
		ball.x += ball.xi * ball.speed;					// Update ball position by speed factor and direction
		ball.y += ball.yi * ball.speed;
		ball.z += ball.zi;
		ball.speed += ball.speedup;						// Slowly increase ball speed by ball-speedup
	}

	// Render Scene //
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);	// Clear screen and depth buffer
	glLoadIdentity();									// Reset the current modelview matrix

	glTranslatef(0, 1, -20.0f);							// Zoom out a bit so we can see the play area
	glRotatef(20, -1, 0, 0);							// Put board on a bit of a lean
	glRotatef(10*float(sin(rot+=0.007f)), 0, 0, 1);		// Swing board around a little

	glEnable(GL_TEXTURE_2D);
	board.Draw();										// Render playing board
	player.Draw();										// Render paddle

	glBindTexture(GL_TEXTURE_2D, ball_texture.ID);
	glEnable(GL_LIGHTING);
	ball.Draw();										// Render ball
	glDisable(GL_LIGHTING);

	glEnable(GL_TEXTURE_2D);							// Reenable texture mapping for font output
	game_system.StartTextMode();						// Go into orthogonal view for easy text output
	glBindTexture(GL_TEXTURE_2D, font.ID);				// Bind font data texture

	glColor4f(1,0.5f,0.5f,1);							// Draw with a bright red colour
	game_system.glPrint(290,460, "Breakout");			// This is the title
	game_system.glPrint(20,20, "Score %d", board.score);// Print the score
	game_system.glPrint(160,20, "Lives %d", player.lives);// Print the lives player has left

	glColor4f(1,0.2f,0.2f,1);							// Draw with a bright red colour
	game_system.glPrint(20,460, "Highscore %d", highscore);
	game_system.glPrint(470,20, "Keys: Left/Right");
	game_system.glPrint(320,20, "ESC to Quit");

	glDisable(GL_BLEND);								// Disable alpha blending
	glColor4f(1,1,1,1);									// White colour

	if (game_state == GET_READY)						
	{
		game_system.glPrint(260, 130, "Get Ready!");	// Print our message when game state is GET_READY
		if (timeGetTime() > game_state_timer)			// If timer has elapsed in GET_READY state
			game_state = PLAYING;						// Then change the game_state to PLAYING
	}

	if (game_state == GAME_OVER)						// Print our message when game state is GAME_OVER
		game_system.glPrint(230, 130, "Press Space To Play");
	game_system.EndTextMode();							// End orthogonal view mode

	glFlush();											// Flush any pending OGL commands out of buffer
	game_system.BufferSwap();							// Swap the display buffers so we can see the result
	return true;
}

//------------------------------------------------------------------//
//- bool Game_Shutdown(void) ---------------------------------------//
//------------------------------------------------------------------//
//- Description: This function does all the necesary shutdown      -//
//-				 procedures in your program/game. It is called once-//
//-              the program recieves a false value from           -//
//-              Game_Main().                                      -//
//------------------------------------------------------------------//

bool Game_Shutdown(void)
{
	game_system.FontShutdown();

	sinput.Shutdown();
	game_system.OpenGLShutdown();

	return false;
}

//------------------------------------------------------------------//
//- int WINAPI WinMain(HINSTANCE, HINSTANCE, LPSTR, int) -----------//
//------------------------------------------------------------------//
//- Description: This is Window's equivalent to Main in dos.  You  -//
//-              don't have to put anything in this function, use  -//
//-              Game_Init(), Game_Main(), and Game_Shutdown() for -//
//-              all of yours needs.                               -//
//------------------------------------------------------------------//

int WINAPI WinMain(HINSTANCE hInstance, 
				   HINSTANCE hPrevInstance, 
				   LPSTR lpCmdLine, 
				   int nCmdShow) 
{
	//Do all of the Initiation stuff
	if (Game_Init()==false)
		return 0;

	while(TRUE)
	{
		if (game_system.HandleMessages()==false)
			break;
		Game_Main();									// Main game loop
	} 

	Game_Shutdown();									// Call Game_Shutdown, and do all the shutdown procedures

	return 0;
}