#include "paddle.h"		// Class declaration

// Constructor

Paddle :: Paddle():
	x(0.0f),
	y(0.0f),
	width(3.0f),
	height(0.5f),
	speed(0.4f),
	lives(3)
{ }

// Destructor

Paddle :: ~Paddle()
{ }

// Loads anything that can't be set at constructor time

Paddle :: LoadData()
{
	// Load the sound effects
	sound_ball_bounce = FSOUND_Sample_Load(FSOUND_FREE, "Data/paddle_bounce.wav", FSOUND_2D, 0);
	// Load the textures
	texture.LoadBMP("Data/paddle.bmp", GL_LINEAR, GL_LINEAR);
}

//------------------------------------------------------------------//
//- void Check_Ball_Collision(Ball *ball) --------------------------//
//------------------------------------------------------------------//
//- Description: This function checks if ball has hit the paddle   -//
//-              and changes ball direction if it happens.		   -//
//------------------------------------------------------------------//

Paddle :: Check_Ball_Collision(Ball *ball)
{
	if ((ball->xi > 0) &&									// If ball is travelling right
		(ball->x < oldx) &&									// Check ball is hasnt past old paddle position, so we dont change direction when the ball is way past the paddle, this takes into account of paddle movement while hitting
		((ball->x + ball->diameter + ball->speed) > x) &&	// Check ball will actually cross the path of the paddle during next move
		((ball->y - ball->diameter) <= y) &&				// Then check we are within the range of the width of the paddle
		(ball->y >= (y - height)))
	{
		ball->xi = -ball->xi;								// Change x direction
		FSOUND_PlaySound(FSOUND_FREE, sound_ball_bounce);	// play "paddle bounce" sound
	}
	else if ((ball->xi < 0) &&								// If ball is travelling left
		(ball->x > (oldx + width)) &&						// Check ball is hasnt past old paddle position, so we dont change direction when the ball is way past the paddle, this takes into account of paddle movement while hitting
		((ball->x - ball->speed) < (x + width)) &&			// Check ball will actually cross the path of the paddle during next move
		((ball->y - ball->diameter) <= y) &&				// Then check we are within the range of the width of the paddle
		(ball->y >= (y - height)))
	{
		ball->xi = -ball->xi;								// Change x direction
		FSOUND_PlaySound(FSOUND_FREE, sound_ball_bounce);	// play "paddle bounce" sound
	}
	else if ((ball->yi < 0) &&								// If ball is travelling down
		((ball->y - ball->diameter > oldy) &&				// Check ball is hasnt past old paddle position, so we dont change direction when the ball is way past the paddle, this takes into account of paddle movement while hitting
		(ball->y - ball->diameter - ball->speed) < y) &&	// Check ball will actually cross the path of the paddle during next move
		((ball->x + ball->diameter) >= x) &&				// Then check we are within the range of the width of the paddle
		(ball->x <= (x + width)))
	{
		// The following code angles the ball depending on where it hits the paddle
		// It just test the proportion from the center of the paddle which is multiplied by 65 degrees
		// This gives a deflection between zero and 65 degrees
		ball->xi = (float)sin( ((ball->x + ball->radius - x - width/2) / (width/2)) * 65/180 * 3.141592654 );
		ball->yi = (float)cos( ((ball->x + ball->radius - x - width/2) / (width/2)) * 65/180 * 3.141592654 );
		FSOUND_PlaySound(FSOUND_FREE, sound_ball_bounce);	// play "paddle bounce" sound
	}
	else if ((ball->yi > 0) &&								// If ball is travelling up
		(ball->y < (y - height)) &&							// Check ball is hasnt past old paddle position, so we dont change direction when the ball is way past the paddle, this takes into account of paddle movement while hitting
		((ball->y + ball->speed) > (y - height)) &&			// Check ball will actually cross the path of the paddle during next move
		((ball->x + ball->diameter) >= x) &&				// Then check we are within the range of the width of the paddle
		(ball->x <= (x + width)))
	{
		ball->yi = -ball->yi;
		FSOUND_PlaySound(FSOUND_FREE, sound_ball_bounce);	// play "paddle bounce" sound
	}
}

// Draw to opengl screen

Paddle::Draw()
{
	glBindTexture(GL_TEXTURE_2D, texture.ID);		// Bind the paddle texture

	glBegin(GL_QUADS);
	// Top
	glColor3f(1,0.2f,0.2f);					// red colour
	  glTexCoord2f(0, 1); glVertex3f(x,			y		  , height);
	  glTexCoord2f(0, 0); glVertex3f(x,			y - height, height);
	glColor3f(0.2f,1.0f,0.2f);				// green colour
	  glTexCoord2f(1, 0); glVertex3f(x + width, y - height, height);
	  glTexCoord2f(1, 1); glVertex3f(x + width, y		  , height);
	// Left side
	glColor3f(0.2f,1.0f,0.2f);				// green colour
	  glVertex3f(x,			y		  , 0.0f);
	  glVertex3f(x,			y - height, 0.0f);
	glColor3f(1,0.2f,0.2f);					// red colour
	  glVertex3f(x,         y - height, height);
	  glVertex3f(x,         y		  , height);
	// Right side
	glColor3f(1,0.2f,0.2f);					// red colour
	  glVertex3f(x + width, y		  , 0.0f);
	  glVertex3f(x + width, y - height, 0.0f);
	glColor3f(0.2f,1.0f,0.2f);				// green colour
	  glVertex3f(x + width, y - height, height);
	  glVertex3f(x + width, y		  , height);
	// Top end
	glColor3f(1,0.2f,0.2f);					// red colour
	  glVertex3f(x + width, y		  , 0.0f);
	glColor3f(0.2f,1.0f,0.2f);				// green colour
	  glVertex3f(x,			y		  , 0.0f);
	glColor3f(1,0.2f,0.2f);					// red colour
	  glVertex3f(x,			y        , height);
	glColor3f(0.2f,1.0f,0.2f);				// green colour
	  glVertex3f(x + width, y		  , height);
	// Bottom end
	glColor3f(0.2f,1.0f,0.2f);				// green colour
	  glVertex3f(x,			y - height, 0.0f);
	glColor3f(1,0.2f,0.2f);					// red colour
	  glVertex3f(x + width, y - height, 0.0f);
	glColor3f(0.2f,1.0f,0.2f);				// green colour
	  glVertex3f(x + width, y - height, height);
	glColor3f(1,0.2f,0.2f);					// red colour
	  glVertex3f(x,			y - height, height);
    glEnd();
}
